package jp.ne.xuni.selene.views;

import java.io.BufferedReader;
import java.io.FileReader;

import org.eclipse.jface.dialogs.IDialogSettings;

import jp.ne.xuni.selene.SelenePlugin;
import jp.ne.xuni.selene.SeleneClient.FetchDocument;
import jp.ne.xuni.selene.ZippedSearchResult;

public class TimeCounter {
	private long startTime;
	private int[] timeRecord;
	private int numOfRecords;
	private int counter;

	private BufferedReader samples;
	private SeleneView view;
	private boolean printRecord;
	private boolean autoTest;

	private IDialogSettings ds = SelenePlugin.getDefault().
		getDialogSettings().getSection(
		SelenePlugin.PLUGIN_ID + ".timecounter");
	{
		if (ds == null) {
			ds = SelenePlugin.getDefault().getDialogSettings().addNewSection(
				SelenePlugin.PLUGIN_ID + ".timecounter");
		}
		String samples = ds.get("samples");
		if (samples != null) {
			try {
				this.samples = new BufferedReader(new FileReader(samples));
			} catch (Exception e) {
				System.err.println("TimeCounter error: " + e.getMessage());
			}
		}
		printRecord = ds.getBoolean("printRecord");
		autoTest = ds.getBoolean("autoTest");
	}

	public TimeCounter(SeleneView view) {
		this.view = view;
	}

	public void setStartTime() {
		startTime = System.currentTimeMillis();
		numOfRecords = 1 + view.numOfSnippets * 3;
		timeRecord = new int[numOfRecords * 2];
		counter = 0;
	}

	public void recordTime() {
		recordTime(-1);
	}
	public void recordTime(int id) {
		if (counter < numOfRecords) {
			timeRecord[counter * 2] = id;
			timeRecord[counter * 2 + 1] =
				(int)(System.currentTimeMillis() - startTime);
			if (counter == numOfRecords - 1) {
				String str = "";
				for (int i = 0; i <= counter * 2 + 1; i++) {
					str += timeRecord[i] + " ";
				}
				if (printRecord) System.out.println(str);

				if (autoTest) {
					try {
						Thread.sleep(500);
					} catch (InterruptedException e) {
						System.err.println(
							"TimeCounter error: " + e.getMessage());
					}
					doNextSearch(); // for automatic evaluation
				}
			}
		}
		counter++;
	}
	private void doNextSearch() {
		try {
			String sample;
			if ((sample = samples.readLine()) != null) {
				FetchDocument job =
					view.client.new FetchDocument("Selene Fetch");
				job.addJobChangeListener(view.new FetchJobListener());
				job.setQuery(new ZippedSearchResult(
					0.0, view.client.getHost(), view.client.getRepository(),
					sample));
				view.launchJob(job);
			}
		} catch (Exception e) {
			System.err.println("TimeCounter error: " + e.getMessage());
		}
	}
}
